# Copyright (c) 2011, Adam Parrish <adam@decontextualize.com>
# 
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
# 
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

class GameState(object):
	def __init__(self):
		self.manager = None
	def add_tag(self, tag):
		self.manager.state_add_tag(self, tag)
	def remove_tag(self, tag):
		self.manager.state_remove_tag(self, tag)

class GameStateManager(object):
	def __init__(self):
		self.states = list()
		self.muted_states = set()
		self.tags = dict()
	def add_state(self, state):
		self.states.append(state)
		state.manager = self
		if hasattr(state, 'state_added'):
			state.state_added()
	def add_state_above(self, under, over):
		assert under in self.states, "can't insert over non-existent state"
		idx = self.states.index(under)
		self.states.insert(idx+1, over)
		over.manager = self
		if hasattr(over, 'state_added'):
			over.state_added()
	def add_state_beneath(self, over, under):
		assert over in self.states, "can't insert beneath non-existent state"
		idx = self.states.index(over)
		self.states.insert(idx, under)
		under.manager = self
		if hasattr(under, 'state_added'):
			under.state_added()
	def remove_state(self, state):
		if state is None: return
		assert state in self.states, "can't remove state that hasn't been added"
		self.states.remove(state)
		if hasattr(state, 'state_removed'):
			state.state_removed()
		for tag, tagged in self.tags.iteritems():
			if state in tagged:
				tagged.remove(state)
	def remove_all(self):
		self.states = []
	def remove_tagged(self, tag):
		to_remove = list()
		if tag in self.tags:
			for state in self.tags[tag]:
				to_remove.append(state)
		for state in to_remove:
			self.remove_state(state)
	def state_add_tag(self, state, tag):
		if tag not in self.tags:
			self.tags[tag] = set()
		self.tags[tag].add(state)
	def state_remove_tag(self, state, tag):
		if tag not in self.tags:
			return
		if state in self.tags[tag]:
			self.tags[tag].remove(state)
	def update(self, delta):
		for state in self.states:
			if state not in self.muted_states and hasattr(state, 'update'):
				state.update(delta)
	def draw(self):
		for state in self.states:
			if state not in self.muted_states and hasattr(state, 'draw'):
				state.draw()
	def mouseClicked(self):
		for state in self.states:
			if state not in self.muted_states and hasattr(state, 'mouseClicked'):
				state.mouseClicked()
	def mousePressed(self):
		for state in self.states:
			if state not in self.muted_states and hasattr(state, 'mousePressed'):
				state.mousePressed()
	def mouseReleased(self):
		for state in self.states:
			if state not in self.muted_states and hasattr(state, 'mouseReleased'):
				state.mouseReleased()
	def keyPressed(self):
		if key == ord('`'):
			print self.states
		for state in self.states:
			if state not in self.muted_states and hasattr(state, 'keyPressed'):
				state.keyPressed()
	def keyReleased(self):
		for state in self.states:
			if state not in self.muted_states and hasattr(state, 'keyReleased'):
				state.keyReleased()
	def mute(self, state):
		self.muted_states.add(state)
	def unmute(self, state):
		if state in self.muted_states:
			self.muted_states.remove(state)
	def remove_instances(self, classes):
		self.states = [state for state in self.states \
			if type(state) not in classes]
	def get_instances(self, classes):
		return [state for state in self.states if type(state) in classes]

class WatcherState(GameState):
	def mouseClicked(self):
		self.manager.add_state(StarfieldState())

class StarfieldState(GameState):
	def __init__(self):
		self.pts = list()
		for i in range(100):
			self.pts.append((random.randrange(800), random.randrange(600)))
	def draw(self):
		for pt in self.pts:
			stroke(255)
			point(pt[0], pt[1])
